@@ -86,6 +86,27 @@ CommandLine.init = function(basePath) {
         blue = "#2E9AFE";
     }
 
+    // open fragment links of format "#<entity ID>,<entity ID>" by selecting
+    // the entities with the given IDs:
+    teHistory.openLinks = false;
+    teHistory.anchorClicked.connect(function(url) {
+        var di = appWin.getDocumentInterface();
+        if (isNull(di)) {
+            return;
+        }
+        if (!url.hasFragment()) {
+            return;
+        }
+
+        var frag = url.fragment();
+        var entityIds = frag.split(",");
+        for (var i=0; i<entityIds.length; i++) {
+            entityIds[i] = parseInt(entityIds[i], 10);
+        }
+
+        di.selectEntities(entityIds);
+    });
+
     function appendAndScroll(msg) {
         teHistory.append(msg);
         var c = teHistory.textCursor();
@@ -237,7 +258,7 @@ CommandLine.init = function(basePath) {
                 pos = stringToCoordinate(di.getRelativeZero(), command);
             }
 
-            if (!isNull(pos)) {
+            if (!pos.isNaN()) {
                 if (!pos.isValid()) {
                     appWin.handleUserWarning(
                         qsTr("Invalid coordinate or distance '%1'.")
@@ -335,39 +356,75 @@ CommandLine.init = function(basePath) {
     
     leCommand.clearHistory.connect(teHistory, "clear");
 
+    leCommand.multiLinePaste.connect(function() {
+        var cb = QGuiApplication.clipboard();
+        qDebug("pasting: " + cb.text());
+
+        var text = cb.text();
+
+        // multi line paste and enter:
+        if (text.contains("\n")) {
+            var lines = text.split("\n");
+            for (var i=0; i<lines.length; i++) {
+                qDebug("line: " + lines[i]);
+                leCommand.commandConfirmed(lines[i]);
+            }
+        }
+        else {
+            // single line paste only:
+            leCommand.paste(text);
+        }
+    });
+
     // show message to user:
-    appWin.userMessage.connect(function(message) {
-            appendAndScroll("<span>" + message + "</span>");
-        });
+    appWin.userMessage.connect(function(message, escape) {
+        if (escape) {
+            message = Qt.escape(message);
+        }
+
+        appendAndScroll("<span>" + message + "</span>");
+    });
 
     // show warning to user:
-    appWin.userWarning.connect(function(message, messageBox) {
-            // TODO: refactor:
-            if (message==="#transaction_failed") {
-                message = qsTr("Transaction failed. Please check for block recursions and locked or invisible layers or blocks.");
-            }
+    appWin.userWarning.connect(function(message, messageBox, escape) {
+        // TODO: refactor:
+        if (message==="#transaction_failed") {
+            message = qsTr("Transaction failed. Please check for block recursions and locked or invisible layers or blocks.");
+        }
 
-            appendAndScroll("<span style='color:#cc0000;'>" + Qt.escape(message) + "</span>");
-            if (RSettings.getBoolValue("CommandLine/WarningsAsDialog", false) || messageBox) {
-                QMessageBox.warning(appWin, qsTr("Warning"), message);
-            }
-        });
+        if (escape) {
+            message = Qt.escape(message);
+        }
+
+        appendAndScroll("<span style='color:#cc0000;'>" + message + "</span>");
+        if (RSettings.getBoolValue("CommandLine/WarningsAsDialog", false) || messageBox) {
+            QMessageBox.warning(appWin, qsTr("Warning"), message);
+        }
+    });
 
     // show info to user:
-    appWin.userInfo.connect(function(message) {
-            appendAndScroll("<span style='color:#0066cc;'>" + Qt.escape(message) + "</span>");
-            if (RSettings.getBoolValue("CommandLine/InfoAsDialog", false)) {
-                QMessageBox.information(appWin, qsTr("Info"), message);
-            }
-        });
+    appWin.userInfo.connect(function(message, escape) {
+        if (escape) {
+            message = Qt.escape(message);
+        }
+
+        appendAndScroll("<span style='color:#0066cc;'>" + message + "</span>");
+        if (RSettings.getBoolValue("CommandLine/InfoAsDialog", false)) {
+            QMessageBox.information(appWin, qsTr("Info"), message);
+        }
+    });
 
     // show previously entered command:
-    appWin.userCommand.connect(function(message) {
+    appWin.userCommand.connect(function(message, escape) {
         // prevent some very frequent commands from being displayed every time:
         if (message==="snapauto" || message==="restrictoff" || message==="escape") {
             return;
         }
 
+        if (escape) {
+            message = Qt.escape(message);
+        }
+
         var cartCoordSep = RSettings.getStringValue("Input/CartesianCoordinateSeparator", ',');
         var polCoordSep = RSettings.getStringValue("Input/PolarCoordinateSeparator", '<');
         var what;
@@ -384,7 +441,7 @@ CommandLine.init = function(basePath) {
         appendAndScroll(
             "<span style='color:" + blue + ";'>"
             + "<i>" + what + ": </i>"
-            + Qt.escape(message) + "</span>");
+            + message + "</span>");
     });
 
     // change command prompt label:
